@@ -41,7 +41,7 @@ module Internal.Util(
     ℕ,ℤ,ℝ,ℂ,iC,
     Normed(..), norm_Frob, norm_nuclear,
     magnit,
-    unitary,
+    normalize,
     mt,
     (~!~),
     pairwiseD2,
@@ -255,6 +255,7 @@ norm :: Vector Double -> Double
 -- ^ 2-norm of real vector
 norm = pnorm PNorm2
 
+-- | p-norm for vectors, operator norm for matrices
 class Normed a
   where
     norm_0   :: a -> R
@@ -319,10 +320,11 @@ instance Normed (Vector (Complex Float))
     norm_2 = norm_2 . double
     norm_Inf = norm_Inf . double
 
-
+-- | Frobenius norm (Schatten p-norm with p=2)
 norm_Frob :: (Normed (Vector t), Element t) => Matrix t -> R
 norm_Frob = norm_2 . flatten
 
+-- | Sum of singular values (Schatten p-norm with p=1)
 norm_nuclear :: Field t => Matrix t -> R
 norm_nuclear = sumElements . singularValues
 
@@ -341,8 +343,8 @@ magnit e x = norm_1 (fromList [x]) > e
 
 
 -- | Obtains a vector in the same direction with 2-norm=1
-unitary :: Vector Double -> Vector Double
-unitary v = v / scalar (norm v)
+normalize :: (Normed (Vector t), Num (Vector t), Field t) => Vector t -> Vector t
+normalize v = v / real (scalar (norm_2 v))
 
 
 -- | trans . inv
